package edu.illinois.cs.cogcomp.LbjNer.LbjTagger;

import java.util.*; 


import edu.illinois.cs.cogcomp.LbjNer.ExpressiveFeatures.BrownClusters;
import edu.illinois.cs.cogcomp.LbjNer.ExpressiveFeatures.Gazzetteers;
import edu.illinois.cs.cogcomp.LbjNer.ExpressiveFeatures.TitleTextNormalizer;
import edu.illinois.cs.cogcomp.LbjNer.ExpressiveFeatures.WikipediaLinkability;
import edu.illinois.cs.cogcomp.LbjNer.ExpressiveFeatures.WordEmbeddings;
import edu.illinois.cs.cogcomp.LbjNer.ExpressiveFeatures.WordEmbeddings.NormalizationMethod;
import edu.illinois.cs.cogcomp.LbjNer.IO.InFile;
import edu.illinois.cs.cogcomp.LbjNer.IO.OutFile;
import edu.illinois.cs.cogcomp.LbjNer.LbjFeatures.NETaggerLevel1;
import edu.illinois.cs.cogcomp.LbjNer.LbjFeatures.NETaggerLevel2;

/**
 * populates a ParametersForLbjCode class's public static members from a config file
 */

public class Parameters {
	public static void readConfigAndLoadExternalData(String configFile) throws Exception{
		ParametersForLbjCode.currentParameters=readAndLoad(configFile);
	}
	
	public static void loadClassifierModels(ParametersForLbjCode config){
		System.out.println("Reading the model at: "+config.pathToModelFile+".level1");
		NETaggerLevel1 taggerLevel1=new NETaggerLevel1(config.pathToModelFile+".level1",config.pathToModelFile+".level1.lex");
		config.taggerLevel1=taggerLevel1;
		System.out.println("Reading the model at: "+config.pathToModelFile+".level2");
		NETaggerLevel2 taggerLevel2=new NETaggerLevel2(config.pathToModelFile+".level2",config.pathToModelFile+".level2.lex");
		config.taggerLevel2=taggerLevel2;
		System.err.println( "## Parameters.loadClassifierModels(): set taggerLevel1 and taggerLevel2 in config passed as argument." );
	}
	
	private static ParametersForLbjCode readAndLoad(String configFile) throws Exception{
		ParametersForLbjCode param=new ParametersForLbjCode();
		param.locationOnDisk=configFile;
		InFile in=new InFile(configFile);

		//reading the next parameter...
		String line=in.readLine();
		StringTokenizer st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("configFilename"))
			throw new Exception("Error- expecting the 'configFilename' parameter, read:"+line);
		param.configFilename=st.nextToken();

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("auxiliaryModels"))
			throw new Exception("Error- expecting the 'auxiliaryModels' parameter, read:"+line);
		while(st.hasMoreTokens()){
			ParametersForLbjCode aux=readAndLoad(st.nextToken());
			aux.nameAsAuxFeature=st.nextToken();
			loadClassifierModels(aux);
			param.auxiliaryModels.addElement(aux);
		}
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("normalizeTitleText"))
			throw new Exception("Error- expecting the 'normalizeTitleText' parameter, read:"+line);
		param.normalizeTitleText=Boolean.parseBoolean(st.nextToken());
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathToTokenNormalizationData"))
			throw new Exception("Error- expecting the 'pathToTokenNormalizationData' parameter, read:"+line);
		param.pathToTokenNormalizationData=st.nextToken();
		TitleTextNormalizer.pathToBrownClusterForWordFrequencies=param.pathToTokenNormalizationData;

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathToLinkabilityData"))
			throw new Exception("Error- expecting the 'pathToLinkabilityData' parameter, read:"+line);
		String pathToLinkabilityData=st.nextToken();

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("linkScoreThres"))
			throw new Exception("Error- expecting the 'linkScoreThres' parameter, read:"+line);
		double linkScoreThres=Double.parseDouble(st.nextToken());

		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("forceNewSentenceOnLineBreaks"))
			throw new Exception("Error- expecting the 'forceNewSentenceOnLineBreaks' parameter, read:"+line);
		param.forceNewSentenceOnLineBreaks=Boolean.parseBoolean(st.nextToken());
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("sortLexicallyFilesInFolders"))
			throw new Exception("Error- expecting the 'sortLexicallyFilesInFolders' parameter, read:"+line);
		param.sortLexicallyFilesInFolders=Boolean.parseBoolean(st.nextToken());

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("treatAllFilesInFolderAsOneBigDocument"))
			throw new Exception("Error- expecting the 'treatAllFilesInFolderAsOneBigDocument' parameter, read:"+line);
		param.treatAllFilesInFolderAsOneBigDocument=Boolean.parseBoolean(st.nextToken());
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("inferenceMethod"))
			throw new Exception("Error- expecting the 'inferenceMethod' parameter, read:"+line);
		param.inferenceMethod=ParametersForLbjCode.InferenceMethods.valueOf(st.nextToken());

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("beamSize"))
			throw new Exception("Error- expecting the 'beamSize' parameter, read:"+line);
		param.beamSize=Integer.parseInt(st.nextToken());		
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("thresholdPrediction"))
			throw new Exception("Error- expecting the 'thresholdPrediction' parameter, read:"+line);
		param.thresholdPrediction=Boolean.parseBoolean(st.nextToken());		
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("predictionConfidenceThreshold"))
			throw new Exception("Error- expecting the 'predictionConfidenceThreshold' parameter, read:"+line);
		param.predictionConfidenceThreshold=Double.parseDouble(st.nextToken());		

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("labelTypes"))
			throw new Exception("Error- expecting the 'labelTypes' parameter, read:"+line);
		Vector<String> types=new Vector<String>();
		while(st.hasMoreTokens())
			types.addElement(st.nextToken());
		param.labelTypes=new String[types.size()];
		for(int i=0;i<param.labelTypes.length;i++)
			param.labelTypes[i]=types.elementAt(i);

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("logging"))
			throw new Exception("Error- expecting the 'logging' parameter, read:"+line);
		param.logging=Boolean.parseBoolean(st.nextToken());

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("debuggingLogPath"))
			throw new Exception("Error- expecting the 'debuggingLogPath' parameter, read:"+line);
		param.debuggingLogPath=st.nextToken()+"/"+param.configFilename+"debugLog.txt";
		if(param.logging)
			param.loggingFile=new OutFile(param.debuggingLogPath);
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("taggingEncodingScheme"))
			throw new Exception("Error- expecting the 'taggingEncodingScheme' parameter, read:"+line);
		String taggingEncodingSchemeString=st.nextToken();
		param.taggingEncodingScheme=TextChunkRepresentationManager.EncodingScheme.valueOf(taggingEncodingSchemeString);
		
		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathToGazetteersLists"))
			throw new Exception("Error- expecting the 'pathToGazetteersLists' parameter, read:"+line);
		String pathToGazetteersLists=st.nextToken();

		
		/*
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathToGazetteersIndex"))
			throw new Exception("Error- expecting the 'pathToGazetteersIndex' parameter, read:"+line);
		String pathToGazetteers=st.nextToken();
		
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathToGazetteersIndexIgnoreCase"))
			throw new Exception("Error- expecting the 'pathToGazetteersIndexIgnoreCase' parameter, read:"+line);		
		String pathToGazetteersIgnoreCase=st.nextToken();
		*/

		//reading the next parameter...
		Vector<String> pathsToBrownClusters=new Vector<String>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathsToBrownClusters"))
			throw new Exception("Error- expecting the 'pathsToBrownClusters' parameter, read:"+line);
		while(st.hasMoreTokens())
			pathsToBrownClusters.addElement(st.nextToken());

		//reading the next parameter...
		Vector<Integer> minWordAppThresholdsForBrownClusters=new Vector<Integer>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("minWordAppThresholdsForBrownClusters"))
			throw new Exception("Error- expecting the 'minWordAppThresholdsForBrownClusters' parameter, read:"+line);
		while(st.hasMoreTokens())
			minWordAppThresholdsForBrownClusters.addElement(Integer.parseInt(st.nextToken()));

		//reading the next parameter...
		Vector<Boolean> lowercaseBrown=new Vector<Boolean>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("isLowercaseBrownClusters"))
			throw new Exception("Error- expecting the 'isLowercaseBrownClusters' parameter, read:"+line);
		while(st.hasMoreTokens())
			lowercaseBrown.addElement(Boolean.parseBoolean(st.nextToken()));

		//reading the next parameter...
		Vector<String> pathsToWordEmbeddings=new Vector<String>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathsToWordEmbeddings"))
			throw new Exception("Error- expecting the 'pathsToWordEmbeddings' parameter, read:"+line);
		while(st.hasMoreTokens())
			pathsToWordEmbeddings.addElement(st.nextToken());

		//reading the next parameter...
		Vector<Integer> dimensionality=new Vector<Integer>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("embeddingDimensionalities"))
			throw new Exception("Error- expecting the 'embeddingDimensionalities' parameter, read:"+line);
		while(st.hasMoreTokens())
			dimensionality.addElement(Integer.parseInt(st.nextToken()));

		//reading the next parameter...
		Vector<Integer> wordAppThresEmbeddings=new Vector<Integer>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("minWordAppThresholdsForEmbeddings"))
			throw new Exception("Error- expecting the 'minWordAppThresholdsForEmbeddings' parameter, read:"+line);
		while(st.hasMoreTokens())
			wordAppThresEmbeddings.addElement(Integer.parseInt(st.nextToken()));

		//reading the next parameter...
		Vector<Double> normalizationConstantsForEmbeddings=new Vector<Double>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("normalizationConstantsForEmbeddings"))
			throw new Exception("Error- expecting the 'normalizationConstantsForEmbeddings' parameter, read:"+line);
		while(st.hasMoreTokens())
			normalizationConstantsForEmbeddings.addElement(Double.parseDouble(st.nextToken()));
		
		//reading the next parameter...
		Vector<NormalizationMethod> normalizationMethodsForEmbeddings=new Vector<NormalizationMethod>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("normalizationMethodsForEmbeddings"))
			throw new Exception("Error- expecting the 'normalizationMethodsForEmbeddings' parameter, read:"+line);
		while(st.hasMoreTokens())
			normalizationMethodsForEmbeddings.addElement(NormalizationMethod.valueOf(st.nextToken()));


		//reading the next parameter...
		Vector<Boolean> isLowercaseWordEmbeddings=new Vector<Boolean>();
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("isLowercaseWordEmbeddings"))
			throw new Exception("Error- expecting the 'isLowercaseWordEmbeddings' parameter, read:"+line);
		while(st.hasMoreTokens())
			isLowercaseWordEmbeddings.addElement(Boolean.parseBoolean(st.nextToken()));

		
		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("pathToModelFile"))
			throw new Exception("Error- expecting the 'pathToModelFile' parameter, read:"+line);
		param.pathToModelFile=st.nextToken()+"/"+param.configFilename+".model";

		//reading the next parameter...
		line=in.readLine();
		st=new StringTokenizer(line,"\t ");
		if(!st.nextToken().equals("tokenizationScheme"))
			throw new Exception("Error- expecting the 'tokenizationScheme' parameter, read:"+line);
		param.tokenizationScheme=ParametersForLbjCode.TokenizationScheme.valueOf(st.nextToken());

		param.featuresToUse=new Hashtable<String,Boolean>();
		line=in.readLine();
		while(line!=null){
			st=new StringTokenizer(line,"\t ");
			String feature=st.nextToken();
			if(st.nextToken().equals("1")){
				System.out.println("Adding feature: "+feature);
				param.featuresToUse.put(feature,true);
			}
			line=in.readLine();
		}
		in.close();
		in.close();

		System.out.println("Working parameters are:");
		System.out.println("inferenceMethod="+param.inferenceMethod.toString());
		System.out.println("beamSize="+param.beamSize);
		System.out.println("thresholdPrediction="+param.thresholdPrediction);
		System.out.println("predictionConfidenceThreshold="+param.predictionConfidenceThreshold);		
		System.out.println("labelTypes");
		for(int i=0;i<param.labelTypes.length;i++)
			System.out.print("\t"+param.labelTypes[i]);
		System.out.println("\nlogging="+param.logging);		
		System.out.println("debuggingLogPath="+param.debuggingLogPath);
		System.out.println("forceNewSentenceOnLineBreaks="+param.forceNewSentenceOnLineBreaks);
		System.out.println("keepOriginalFileTokenizationAndSentenceSplitting="+param.keepOriginalFileTokenizationAndSentenceSplitting);
		System.out.println("taggingScheme="+param.taggingEncodingScheme.toString());
		System.out.println("tokenizationScheme="+param.tokenizationScheme.toString());
		System.out.println("pathToModelFile="+param.pathToModelFile);
		if(param.featuresToUse.containsKey("BrownClusterPaths")){
			for(int i=0;i<pathsToBrownClusters.size();i++){
				System.out.println("Brown clusters resource: ");
				System.out.println("\t-Path: "+pathsToBrownClusters.elementAt(i));
				System.out.println("\t-WordThres="+minWordAppThresholdsForBrownClusters.elementAt(i));
				System.out.println("\t-IsLowercased="+lowercaseBrown.elementAt(i));
			}
		}
		if(param.featuresToUse.containsKey("WordEmbeddings")){
			for(int i=0;i<pathsToWordEmbeddings.size();i++){
				System.out.println("Words Embeddings resource: ");
				System.out.println("\t-Path: "+pathsToWordEmbeddings.elementAt(i));
				System.out.println("\t-Dimensionality="+dimensionality.elementAt(i));
				System.out.println("\t-WordThres="+wordAppThresEmbeddings.elementAt(i));
				System.out.println("\t-IsLowercased="+isLowercaseWordEmbeddings.elementAt(i));
			}			
		}
		
		
		if(param.featuresToUse.containsKey("BrownClusterPaths"))
			BrownClusters.init(pathsToBrownClusters,minWordAppThresholdsForBrownClusters,lowercaseBrown);
		if(param.featuresToUse.containsKey("WordEmbeddings"))
			WordEmbeddings.init(pathsToWordEmbeddings,dimensionality,wordAppThresEmbeddings,isLowercaseWordEmbeddings,normalizationConstantsForEmbeddings,normalizationMethodsForEmbeddings);
		if(param.featuresToUse.containsKey("GazetteersFeatures")) {
			// System.out.println("Loading the gazetteers from the files:\n\t "+pathToGazetteersIndex+"\nand\n\t"+pathToGazetteersIndexIgnoreCase);
			Gazzetteers.init(pathToGazetteersLists);//pathToGazetteers, pathToGazetteersIgnoreCase);
		}
		if(param.featuresToUse.containsKey("Linkability"))
			WikipediaLinkability.init(pathToLinkabilityData, linkScoreThres);

		param.featuresToUse.put("TitleNormalization", true);
		param.featuresToUse.put("WordTopicTitleInfo", true);

		
		//don't forget that these should be initialized only after we know the target labels and the encoding scheme
		param.patternLabelRandomGenerator=new RandomLabelGenerator(param.labelTypes,param.taggingEncodingScheme);
		param.level1AggregationRandomGenerator=new RandomLabelGenerator(param.labelTypes,param.taggingEncodingScheme);
		param.prevPredictionsLevel1RandomGenerator=new RandomLabelGenerator(param.labelTypes,param.taggingEncodingScheme);
		param.prevPredictionsLevel2RandomGenerator=new RandomLabelGenerator(param.labelTypes,param.taggingEncodingScheme);
		
		return param;
	}	
}
